@@ -830,7 +830,7 @@ static netdev_tx_t zd1201_hard_start_xmit(struct sk_buff *skb,
 	return NETDEV_TX_OK;
 }
 
-static void zd1201_tx_timeout(struct net_device *dev)
+static void zd1201_tx_timeout(struct net_device *dev, unsigned int txqueue)
 {
 	struct zd1201 *zd = netdev_priv(dev);
 
@@ -843,6 +843,14 @@ static void zd1201_tx_timeout(struct net_device *dev)
 	/* Restart the timeout to quiet the watchdog: */
 	netif_trans_update(dev); /* prevent tx timeout */
 }
+#if LINUX_VERSION_IS_LESS(5,6,0)
+/* Just declare it here to keep sparse happy */
+void bp_zd1201_tx_timeout(struct net_device *dev);
+void bp_zd1201_tx_timeout(struct net_device *dev) {
+	zd1201_tx_timeout(dev, 0);
+}
+EXPORT_SYMBOL_GPL(bp_zd1201_tx_timeout);
+#endif
 
 static int zd1201_set_mac_address(struct net_device *dev, void *p)
 {
@@ -1652,15 +1660,11 @@ static int zd1201_set_maxassoc(struct net_device *dev,
     struct iw_request_info *info, struct iw_param *rrq, char *extra)
 {
 	struct zd1201 *zd = netdev_priv(dev);
-	int err;
 
 	if (!zd->ap)
 		return -EOPNOTSUPP;
 
-	err = zd1201_setconfig16(zd, ZD1201_RID_CNFMAXASSOCSTATIONS, rrq->value);
-	if (err)
-		return err;
-	return 0;
+	return zd1201_setconfig16(zd, ZD1201_RID_CNFMAXASSOCSTATIONS, rrq->value);
 }
 
 static int zd1201_get_maxassoc(struct net_device *dev,
@@ -1718,7 +1722,12 @@ static const struct net_device_ops zd1201_netdev_ops = {
 	.ndo_open		= zd1201_net_open,
 	.ndo_stop		= zd1201_net_stop,
 	.ndo_start_xmit		= zd1201_hard_start_xmit,
+#if LINUX_VERSION_IS_GEQ(5,6,0)
 	.ndo_tx_timeout		= zd1201_tx_timeout,
+#else
+	.ndo_tx_timeout = bp_zd1201_tx_timeout,
+#endif
+
 	.ndo_set_rx_mode	= zd1201_set_multicast,
 	.ndo_set_mac_address	= zd1201_set_mac_address,
 	.ndo_validate_addr	= eth_validate_addr,
